/*
 *  Copyright (c) 1990 Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  unbind.c
 */

#ifndef lint 
static char copyright[] = "@(#) Copyright (c) 1990 Regents of the University of Michigan.\nAll rights reserved.\n";
#endif

#include <stdio.h>
#include <string.h>
#ifdef MACOS
#include <stdlib.h>
#include "macos.h"
#else /* MACOS */
#if defined( DOS ) || defined( _WIN32 )
#include "msdos.h"
#ifdef NCSA
#include "externs.h"
#endif /* NCSA */
#else /* DOS */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#endif /* DOS */
#endif /* MACOS */

#include "lber.h"
#include "ldap.h"

#ifdef MACOS
#define tcp_close( s )		tcpclose( s )
#else /* MACOS */
#ifdef DOS
#ifdef PCNFS
#define tcp_close( s )		close( s )
#endif /* PCNFS */
#ifdef NCSA
#define tcp_close( s )		netclose( s ); netshut()
#endif /* NCSA */
#ifdef WINSOCK
#define tcp_close( s )		closesocket( s ); WSACleanup();
#endif /* WINSOCK */
#else /* DOS */
#define tcp_close( s )		close( s )
#endif /* DOS */
#endif /* MACOS */

int
ldap_unbind( LDAP *ld )
{
	BerElement	*ber;
	LDAPMessage	*lm, *next;
	int		err = 0;

	Debug( LDAP_DEBUG_TRACE, "ldap_unbind\n", 0, 0, 0 );

	if ( ld->ld_sb.sb_naddr == 0 ) {
		/* create a message to send */
		if ( (ber = ber_alloc_t( ld->ld_lberoptions )) == NULLBER ) {
			ld->ld_errno = LDAP_NO_MEMORY;
			err = -1;
		}

		/* fill it in */
		if ( !err && ber_printf( ber, "{itn}", ++ld->ld_msgid,
		    LDAP_REQ_UNBIND ) == -1 ) {
			ld->ld_errno = LDAP_ENCODING_ERROR;
			ber_free( ber, 1 );
			err = -1;
		}

		/* send the message */
		if ( !err && ber_flush( &ld->ld_sb, ber, 1 ) == -1 ) {
			ld->ld_errno = LDAP_SERVER_DOWN;
			ber_free( ber, 1 );
			err = -1;
		}
	}

	/* free LDAP structure and outstanding requests/responses */
	for ( lm = ld->ld_requests; lm != NULL; lm = next ) {
		next = lm->lm_next;
		ldap_msgfree( lm );
	}

	for ( lm = ld->ld_responses; lm != NULL; lm = next ) {
		next = lm->lm_next;
		ldap_msgfree( lm );
	}

	if ( ld->ld_sb.sb_naddr == 0 ) {
		tcp_close( ld->ld_sb.sb_sd );
	} else {
		int	i;

		for ( i = 0; i < ld->ld_sb.sb_naddr; ++i ) {
			free( ld->ld_sb.sb_addrs[ i ] );
		}
		free( ld->ld_sb.sb_addrs );
		free( ld->ld_sb.sb_fromaddr );
	}

#ifndef NO_CACHE
	if ( ld->ld_cache != NULL )
		ldap_destroy_cache( ld );
#endif /* !NO_CACHE */
	if ( ld->ld_error != NULL )
		free( ld->ld_error );
	if ( ld->ld_matched != NULL )
		free( ld->ld_matched );
	if ( ld->ld_host != NULL )
		free( ld->ld_host );
	if ( ld->ld_ufnprefix != NULL )
		free( ld->ld_ufnprefix );
	if ( ld->ld_filtd != NULL )
		ldap_getfilter_free( ld->ld_filtd );
	if ( ld->ld_sb.sb_ber.ber_buf != NULL )
		free( ld->ld_sb.sb_ber.ber_buf );
	if ( ld->ld_abandoned != NULL )
		free( ld->ld_abandoned );

	free( (char *) ld );

	return( err );
}

int
ldap_unbind_s( LDAP *ld )
{
	return( ldap_unbind( ld ) );
}

